import {
  scaleLinear,
  extent,
  axisLeft,
  axisBottom,
  transition,
} from 'd3';

export const scatterPlot = () => {
  let width;
  let height;
  let margin;
  let data;
  let radius;
  let xValue;
  let yValue;
  let xAxisTitle;
  let yAxisTitle;

  const my = (selection) => {
    const x = scaleLinear()
      .domain(extent(data, xValue))
      .range([margin.left, width - margin.right]);

    const y = scaleLinear()
      .domain(extent(data, yValue))
      .range([height - margin.bottom, margin.top]);

    const marks = data.map((d) => ({
      x: x(xValue(d)),
      y: y(yValue(d)),
    }));

    const xAxis = axisBottom().scale(x);
    const yAxis = axisLeft().scale(y);

    const t = transition().duration(1000);

    selection
      .selectAll('circle')
      .data(marks)
      .join(
        (enter) =>
          enter
            .append('circle')
            .attr('cx', (d) => d.x)
            .attr('cy', (d) => d.y)
            .attr('r', 0)
            .call((enter) =>
              enter.transition(t).attr('r', radius)
            ),
        (update) =>
          update.call((update) =>
            update
              .transition(t)
              .delay((d, i) => i * 10)
              .attr('cx', (d) => d.x)
              .attr('cy', (d) => d.y)
          ),
        (exit) => exit.remove()
      );

    selection
      .selectAll('.x-axis')
      .data([null])
      .join('g')
      .attr('class', 'x-axis')
      .attr(
        'transform',
        `translate(0, ${height - margin.bottom})`
      )
      .transition(t)
      .call(xAxis);

    selection
      .selectAll('.y-axis')
      .data([null])
      .join('g')
      .attr('class', 'y-axis')
      .attr('transform', `translate(${margin.left}, 0)`)
      .transition(t)
      .call(yAxis);

    selection
      .selectAll('.axis-title-x')
      .data([null])
      .join(
        (enter) =>
          enter
            .append('text')
            .text(xAxisTitle)
            .attr('class', 'axis-title-x')
            .attr('x', width / 2)
            .attr('y', height * 1.5)
            .attr('text-anchor', 'middle')
            .call((enter) =>
              enter
                .transition(t)
                .attr('y', height - margin.bottom / 2.5)
            ),
        (update) =>
          update.call((update) =>
            update
              .attr('y', height * 1.5)
              .transition(t)
              .text(xAxisTitle)
              .attr('y', height - margin.bottom / 2.5)
          )
      );

    selection
      .selectAll('.axis-title-y')
      .data([null])
      .join('text')
      .attr('class', 'axis-title-y')
      .text(yAxisTitle)
      .attr('transform', 'rotate(-90)')
      .attr('y', 0)
      .attr('x', -height / 2)
      .attr('dy', '1em');
  };

  // Getters and setters
  my.width = function (_) {
    return arguments.length ? ((width = _), my) : width;
  };

  my.height = function (_) {
    return arguments.length ? ((height = _), my) : height;
  };

  my.margin = function (_) {
    return arguments.length ? ((margin = _), my) : margin;
  };

  my.data = function (_) {
    return arguments.length ? ((data = _), my) : data;
  };

  my.radius = function (_) {
    return arguments.length ? ((radius = _), my) : radius;
  };

  my.xValue = function (_) {
    return arguments.length ? ((xValue = _), my) : xValue;
  };

  my.yValue = function (_) {
    return arguments.length ? ((yValue = _), my) : yValue;
  };

  my.xAxisTitle = function (_) {
    return arguments.length
      ? ((xAxisTitle = _), my)
      : xAxisTitle;
  };

  my.yAxisTitle = function (_) {
    return arguments.length
      ? ((yAxisTitle = _), my)
      : yAxisTitle;
  };

  return my;
};
